#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfiguratorTest.py
# unittests for field Tags running Tango Server
#
import unittest
import os
import sys
import random
import struct
import binascii

try:
    import tango
except Exception:
    import PyTango as tango

import nxstools
from nxstools import nxscreate
from nxstools import nxsdevicetools

try:
    import nxsextrasp00
except ImportError:
    from . import nxsextrasp00

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

try:
    import TestServerSetUp
except ImportError:
    from . import TestServerSetUp


if sys.version_info > (3,):
    unicode = str
    long = int


# if 64-bit machione
IS64BIT = (struct.calcsize("P") == 8)


# test fixture
class NXSCreateOnlineDSFSTest(unittest.TestCase):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        unittest.TestCase.__init__(self, methodName)

        try:
            # random seed
            self.seed = long(binascii.hexlify(os.urandom(16)), 16)
        except NotImplementedError:
            import time
            # random seed
            self.seed = long(time.time() * 256)  # use fractional seconds

        self._rnd = random.Random(self.seed)

        self._bint = "int64" if IS64BIT else "int32"
        self._buint = "uint64" if IS64BIT else "uint32"
        self._bfloat = "float64" if IS64BIT else "float32"

        self.__args = '{"db":"nxsconfig", ' \
                      '"read_default_file":"/etc/my.cnf", "use_unicode":true}'

        # home = expanduser("~")
        self.maxDiff = None
        db = tango.Database()
        self.host = db.get_db_host().split(".")[0]
        self.port = db.get_db_port()
        self.directory = "."
        self.flags = "-d . "

    # sets xmlconfiguration
    # \param xmlc configuration instance
    # \param xml xml configuration string
    def setXML(self, xmlc, xml):
        xmlc.XMLString = xml

    # gets xmlconfiguration
    # \param xmlc configuration instance
    # \returns xml configuration string
    def getXML(self, xmlc):
        return xmlc.XMLString

    # test starter
    # \brief Common set up
    def setUp(self):
        print("\nsetting up...")
        print("SEED = %s" % self.seed)

    # test closer
    # \brief Common tear down
    def tearDown(self):
        print("tearing down ...")

    def dsexists(self, name):
        return os.path.isfile("%s/%s.ds.xml" % (self.directory, name))

    def cpexists(self, name):
        return os.path.isfile("%s/%s.xml" % (self.directory, name))

    def getds(self, name):
        with open("%s/%s.ds.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def getcp(self, name):
        with open("%s/%s.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def deleteds(self, name):
        os.remove("%s/%s.ds.xml" % (self.directory, name))

    def deletecp(self, name):
        os.remove("%s/%s.xml" % (self.directory, name))

    def runtest(self, argv):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        err = False
        try:
            nxscreate.main()
        except Exception:
            err = True
        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        if err:
            print(vl)
            print(er)
            raise
        return vl, er

    def runtestexcept(self, argv, exception):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        try:
            error = False
            nxscreate.main()
        except exception:
            error = True
        self.assertEqual(error, True)

        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    # Exception tester
    # \param exception expected exception
    # \param method called method
    # \param args list with method arguments
    # \param kwargs dictionary with method arguments
    def myAssertRaise(self, exception, method, *args, **kwargs):
        try:
            error = False
            method(*args, **kwargs)
        except exception:
            error = True
        self.assertEqual(error, True)

    def test_onlineds_stepping_motor(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
</hw>
"""

        args = [
            [
                ('nxscreate onlineds %s %s'
                 % (fname, self.flags)).split(),
                ['my_exp_mot01',
                 'my_exp_mot02',
                 'my_exp_mot03'],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot01">\n'
                    '    <device name="p09/motor/exp.01"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot02">\n'
                    '    <device name="p09/motor/exp.02"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot03">\n'
                    '    <device name="p09/motor/exp.03"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
        ]

        totest = []
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    if er:
                        self.assertTrue(er.startswith(
                            "Info"))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            os.remove(fname)
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_stepping_motor_noclientlike(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_oh1_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/oh1.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_oh1</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/oh1</rootdevicename>
</device>
<device>
 <name>my_oh1_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/oh1.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_oh1</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/oh1</rootdevicename>
</device>
<device>
 <name>my_oh1_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/oh1.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_oh1</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/oh1</rootdevicename>
</device>
</hw>
"""

        args = [
            [
                ('nxscreate onlineds -t %s %s'
                 % (fname, self.flags)).split(),
                ['my_oh1_mot01',
                 'my_oh1_mot02',
                 'my_oh1_mot03'],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_oh1_mot01">\n'
                    '    <device name="p09/motor/oh1.01"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_oh1_mot02">\n'
                    '    <device name="p09/motor/oh1.02"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_oh1_mot03">\n'
                    '    <device name="p09/motor/oh1.03"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
            [
                ('nxscreate onlineds --noclientlike %s %s'
                 % (fname, self.flags)).split(),
                ['my_oh1_mot01',
                 'my_oh1_mot02',
                 'my_oh1_mot03'],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_oh1_mot01">\n'
                    '    <device name="p09/motor/oh1.01"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_oh1_mot02">\n'
                    '    <device name="p09/motor/oh1.02"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_oh1_mot03">\n'
                    '    <device name="p09/motor/oh1.03"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
        ]

        totest = []
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    if er:
                        self.assertTrue(er.startswith(
                            "Info"))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            os.remove(fname)
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_stepping_motor_lower(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/Motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_Mot02</name>
 <type>Stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>My_exp_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
</hw>
"""

        args = [
            [
                ('nxscreate onlineds %s %s'
                 % (fname, self.flags)).split(),
                ['my_exp_mot01',
                 'my_exp_mot02',
                 'my_exp_mot03'],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot01">\n'
                    '    <device name="p09/motor/exp.01"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot02">\n'
                    '    <device name="p09/motor/exp.02"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot03">\n'
                    '    <device name="p09/motor/exp.03"'
                    ' member="attribute" hostname="haso000" '
                    'port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
        ]

        totest = []
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    if er:
                        self.assertTrue(er.startswith(
                            "Info"))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            os.remove(fname)
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_stepping_motor_nolower(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/Motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_Mot02</name>
 <type>Stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>My_exp_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
</hw>
"""

        args = [
            [
                ('nxscreate onlineds %s %s -n '
                 % (fname, self.flags)).split(),
                ['my_exp_mot01',
                 'my_exp_Mot02',
                 'My_exp_mot03'],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot01">\n'
                    '    <device name="p09/Motor/exp.01" '
                    'member="attribute" '
                    'hostname="haso000" port="10000" '
                    'group="__CLIENT__"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_Mot02">\n'
                    '    <device name="P09/motor/exp.02"'
                    ' member="attribute" '
                    'hostname="haso000" port="10000" '
                    'group="__CLIENT__"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="My_exp_mot03">\n'
                    '    <device name="P09/motor/exp.03"'
                    ' member="attribute" '
                    'hostname="haso000" port="10000" '
                    'group="__CLIENT__"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
            [
                ('nxscreate onlineds %s %s --nolower '
                 % (fname, self.flags)).split(),
                ['my_exp_mot01',
                 'my_exp_Mot02',
                 'My_exp_mot03'],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_mot01">\n'
                    '    <device name="p09/Motor/exp.01"'
                    ' member="attribute" '
                    'hostname="haso000" port="10000" '
                    'group="__CLIENT__"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_exp_Mot02">\n'
                    '    <device name="P09/motor/exp.02"'
                    ' member="attribute" '
                    'hostname="haso000" port="10000" '
                    'group="__CLIENT__"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="My_exp_mot03">\n'
                    '    <device name="P09/motor/exp.03"'
                    ' member="attribute" '
                    'hostname="haso000" port="10000" '
                    'group="__CLIENT__"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
        ]

        totest = []
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    if er:
                        self.assertTrue(er.startswith(
                            "Info"))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            os.remove(fname)
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_motors(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
    <name>my_abs</name>
    <type>step_motor</type>
    <module>absbox</module>
    <device>p09/absorbercontroller/mag.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my2_abs</name>
    <type>motor</type>
    <module>absbox</module>
    <device>p09/absorbercontroller/mag.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_tango_motor</name>
    <type>motor</type>
    <module>motor_tango</module>
    <device>p09/motor/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_kohzu</name>
    <type>motor</type>
    <module>kohzu</module>
    <device>p09/kohzu/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_smchydra</name>
    <type>motor</type>
    <module>smchydra</module>
    <device>p09/smchydra/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_lom</name>
    <type>motor</type>
    <module>lom</module>
    <device>p09/lom/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_oms58</name>
    <type>motor</type>
    <module>oms58</module>
    <device>p09/oms58/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_e6c</name>
    <type>motor</type>
    <module>e6c</module>
    <device>p09/e6c/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_omsmaxv</name>
    <type>motor</type>
    <module>omsmaxv</module>
    <device>p09/omsmaxv/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_spk</name>
    <type>motor</type>
    <module>spk</module>
    <device>p09/spk/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_pie710</name>
    <type>motor</type>
    <module>pie710</module>
    <device>p09/pie710/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_pie712</name>
    <type>motor</type>
    <module>pie712</module>
    <device>p09/pie712/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_e6c_p09_eh2</name>
    <type>motor</type>
    <module>e6c_p09_eh2</module>
    <device>p09/e6c_p09_eh2/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_smaract</name>
    <type>motor</type>
    <module>smaract</module>
    <device>p09/smaract/mot.01</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""

        args = [
            [
                ('nxscreate onlineds %s %s'
                 % (fname, self.flags)).split(),
                [
                    'my_abs',
                    'my2_abs',
                    'my_tango_motor',
                    'my_kohzu',
                    'my_smchydra',
                    'my_lom',
                    'my_oms58',
                    'my_e6c',
                    'my_omsmaxv',
                    'my_spk',
                    'my_pie710',
                    'my_pie712',
                    'my_e6c_p09_eh2',
                    'my_smaract',
                ],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_abs">\n'
                    '    <device name="p09/absorbercontroller/mag.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my2_abs">\n'
                    '    <device name="p09/absorbercontroller/mag.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_tango_motor">\n'
                    '    <device name="p09/motor/mot.01" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_kohzu">\n'
                    '    <device name="p09/kohzu/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_smchydra">\n'
                    '    <device name="p09/smchydra/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_lom">\n'
                    '    <device name="p09/lom/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_oms58">\n'
                    '    <device name="p09/oms58/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_e6c">\n'
                    '    <device name="p09/e6c/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_omsmaxv">\n'
                    '    <device name="p09/omsmaxv/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_spk">\n'
                    '    <device name="p09/spk/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_pie710">\n'
                    '    <device name="p09/pie710/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_pie712">\n'
                    '    <device name="p09/pie712/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_e6c_p09_eh2">\n'
                    '    <device name="p09/e6c_p09_eh2/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_smaract">\n'
                    '    <device name="p09/smaract/mot.01"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Position"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
        ]

        totest = []
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    if er:
                        self.assertTrue(er.startswith(
                            "Info"))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            os.remove(fname)
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_attributes(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
    <name>my_test_counter_tango</name>
    <type>type_tango</type>
    <module>counter_tango</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_v260</name>
    <type>type_tango</type>
    <module>v260</module>
    <device>mytest/v260/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>

<device>
    <name>my_test_dgg2</name>
    <type>type_tango</type>
    <module>dgg2</module>
    <device>mytest/dgg2/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_mca_8701</name>
    <type>type_tango</type>
    <module>mca_8701</module>
    <device>mytest/mca_8701/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_mca_sis3302new</name>
    <type>type_tango</type>
    <module>mca_sis3302new</module>
    <device>mytest/mca_sis3302new/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_mca_sis3302</name>
    <type>type_tango</type>
    <module>mca_sis3302</module>
    <device>mytest/mca_sis3302/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_sis3610</name>
    <type>type_tango</type>
    <module>sis3610</module>
    <device>mytest/sis3610/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_vdot32in</name>
    <type>type_tango</type>
    <module>vdot32in</module>
    <device>mytest/vdot32in/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_sis3820</name>
    <type>type_tango</type>
    <module>sis3820</module>
    <device>mytest/sis3820/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip551</name>
    <type>type_tango</type>
    <module>tip551</module>
    <device>mytest/tip551/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip850dac</name>
    <type>type_tango</type>
    <module>tip850dac</module>
    <device>mytest/tip850dac/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip830</name>
    <type>type_tango</type>
    <module>tip830</module>
    <device>mytest/tip830/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip850adc</name>
    <type>type_tango</type>
    <module>tip850adc</module>
    <device>mytest/tip850adc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_vfcadc</name>
    <type>type_tango</type>
    <module>vfcadc</module>
    <device>mytest/vfcadc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""

        args = [
            [
                ('nxscreate onlineds %s %s'
                 % (fname, self.flags)).split(),
                [
                    'my_test_counter_tango',
                    'my_test_v260',
                    'my_test_dgg2',
                    'my_test_mca_8701',
                    'my_test_mca_sis3302new',
                    'my_test_mca_sis3302',
                    'my_test_sis3610',
                    'my_test_vdot32in',
                    'my_test_sis3820',
                    'my_test_tip551',
                    'my_test_tip850dac',
                    'my_test_tip830',
                    'my_test_tip850adc',
                    'my_test_vfcadc',
                ],
                [
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource'
                    ' type="TANGO" name="my_test_counter_tango">\n'
                    '    <device name="mytest/vcexecutor/ct"'
                    ' member="attribute" hostname="haso000"'
                    ' port="10000" group="__CLIENT__"'
                    '/>\n    <record name="Counts"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_v260">\n'
                    '    <device name="mytest/v260/ct" member="attribute"'
                    ' hostname="haso000"'
                    ' port="10000" group="__CLIENT__"/>\n'
                    '    <record name="Counts"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_dgg2">\n'
                    '    <device name="mytest/dgg2/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"'
                    '/>\n    <record name="SampleTime"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_mca_8701">\n'
                    '    <device name="mytest/mca_8701/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Data"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO"'
                    ' name="my_test_mca_sis3302new">\n'
                    '    <device name="mytest/mca_sis3302new/ct"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Data"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_mca_sis3302">\n'
                    '    <device name="mytest/mca_sis3302/ct"'
                    ' member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Data"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_sis3610">\n'
                    '    <device name="mytest/sis3610/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Value"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_vdot32in">\n'
                    '    <device name="mytest/vdot32in/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Value"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_sis3820">\n'
                    '    <device name="mytest/sis3820/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Counts"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_tip551">\n'
                    '    <device name="mytest/tip551/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Voltage"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_tip850dac">\n'
                    '    <device name="mytest/tip850dac/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Voltage"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_tip830">\n'
                    '    <device name="mytest/tip830/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Counts"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_tip850adc">\n'
                    '    <device name="mytest/tip850adc/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Value"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                    '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                    '<definition>\n'
                    '  <datasource type="TANGO" name="my_test_vfcadc">\n'
                    '    <device name="mytest/vfcadc/ct" member="attribute"'
                    ' hostname="haso000" port="10000"'
                    ' group="__CLIENT__"/>\n    <record name="Counts"/>\n'
                    '  </datasource>\n'
                    '</definition>\n',
                ],
            ],
        ]

        totest = []
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    if er:
                        self.assertTrue(er.startswith(
                            "Info"))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            os.remove(fname)
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_attributes_alias(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = '<?xml version="1.0"?>\n' \
              '<hw>\n' \
              '<device>\n' \
              '    <name>%s</name>\n' \
              '    <type>type_tango</type>\n' \
              '    <module>counter_tango</module>\n' \
              '    <device>%s</device>\n' \
              '    <control>tango</control>\n' \
              '    <hostname>haso000:10000</hostname>\n' \
              '</device>\n' \
              '</hw>\n'

        dsout = \
            '<?xml version=\'1.0\' encoding=\'utf8\'?>\n' \
            '<definition>\n' \
            '  <datasource name="%s"' \
            ' type="TANGO">\n' \
            '    <device group="__CLIENT__" hostname="%s"' \
            ' member="attribute" name="%s"' \
            ' port="%s"/>\n    <record name="%s"/>\n' \
            '  </datasource>\n' \
            '</definition>\n'

        command = ('nxscreate onlineds %s %s'
                   % (fname, self.flags)).split()

        args = [
            ['my_test_%s' % ky, "mytest/%s/00" % ky, vl[0]]
            for ky, vl in nxsdevicetools.moduleAttributes.items()
            if ky not in nxsdevicetools.moduleAttributes.keys()
        ]

        totest = []
        try:
            for arg in args:
                ds = arg[0]
                dv = arg[1]
                attr = arg[2]
                print("%s %s %s " % (ds, dv, attr))
                if os.path.isfile(fname):
                    raise Exception("Test file %s exists" % fname)
                print(xml % (ds, dv))
                with open(fname, "w") as fl:
                    fl.write(xml % (ds, dv))
                try:
                    tsv = TestServerSetUp.TestServerSetUp(
                        dv, "MYTESTS1",
                        ds
                    )
                    tsv.setUp()
                    skip = False
                    if self.dsexists(ds):
                        skip = True
                    if not skip:
                        totest.append(ds)

                        vl, er = self.runtest(command)

                        if er:
                            self.assertTrue(er.startswith(
                                "Info"))
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)

                        dsxml = self.getds(ds)
                        self.assertEqual(
                            dsout % (ds, self.host, dv, self.port, attr),
                            dsxml)

                        self.deleteds(ds)
                finally:
                    os.remove(fname)
                    if tsv:
                        tsv.tearDown()
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_attributes_multi(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = '<?xml version="1.0"?>\n' \
              '<hw>\n' \
              '<device>\n' \
              '    <name>%s</name>\n' \
              '    <type>type_tango</type>\n' \
              '    <module>%s</module>\n' \
              '    <device>%s</device>\n' \
              '    <control>tango</control>\n' \
              '    <hostname>%s:%s</hostname>\n' \
              '</device>\n' \
              '</hw>\n'

        dsout = \
            '<?xml version=\'1.0\' encoding=\'utf8\'?>\n' \
            '<definition>\n' \
            '  <datasource type="TANGO" name="%s">\n' \
            '    <device name="%s" member="attribute" hostname="%s"' \
            ' port="%s" group="%s"/>\n' \
            '    <record name="%s"/>\n' \
            '  </datasource>\n' \
            '</definition>\n'

        command = ('nxscreate onlineds %s %s'
                   % (fname, self.flags)).split()

        args = [
            ['my_test_%s' % ky, "mytest/%s/00" % ky, vl, ky]
            for ky, vl in nxstools.xmltemplates.moduleMultiAttributes.items()
        ]

        totest = []
        try:
            for arg in args:
                ds = arg[0]
                dv = arg[1]
                attr = list(arg[2])
                module = arg[3]
                if ("%s@pool" % module) in \
                   nxstools.xmltemplates.moduleMultiAttributes.keys():
                    attr.extend(
                        nxstools.xmltemplates.moduleMultiAttributes[
                            "%s@pool" % module])
                    attr.append("")
                if os.path.isfile(fname):
                    raise Exception("Test file %s exists" % fname)
                with open(fname, "w") as fl:
                    fl.write(xml % (ds, module, dv, self.host, self.port))
                try:
                    tsv = TestServerSetUp.TestServerSetUp(
                        dv, "MYTESTS1",
                        ds
                    )
                    tsv.setUp()
                    # for el in attr:
                    #     if el not in [""]:
                    #         try:
                    #             tsv.dp.CreateAttribute(el)
                    #         except Exception as e:
                    #             print(e)
                    #             tsv.dp.CreateAttribute(el)

                    skip = False
                    for el in attr:
                        if self.dsexists(
                                "%s_%s" % (ds, el.lower())
                                if el else ds):
                            skip = True
                    if not skip:
                        for el in attr:
                            totest.append(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )
                        vl, er = self.runtest(command)

                        if er:
                            self.assertTrue(er.startswith(
                                "Info"))
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)

                        for el in attr:
                            dsxml = self.getds(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )
                            self.assertEqual(
                                dsout % (
                                    "%s_%s" % (ds, el.lower()) if el else ds,
                                    dv,
                                    self.host, self.port,
                                    "%s_" % (ds) if el else "__CLIENT__",
                                    el or "Value"),
                                dsxml)

                        for el in attr:
                            self.deleteds(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )
                finally:
                    os.remove(fname)
                    if tsv:
                        tsv.tearDown()
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_attributes_multi_xmltemplates(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = '<?xml version="1.0"?>\n' \
              '<hw>\n' \
              '<device>\n' \
              '    <name>%s</name>\n' \
              '    <type>type_tango</type>\n' \
              '    <module>%s</module>\n' \
              '    <device>%s</device>\n' \
              '    <control>tango</control>\n' \
              '    <hostname>%s:%s</hostname>\n' \
              '</device>\n' \
              '</hw>\n'

        dsout = \
            '<?xml version=\'1.0\' encoding=\'utf8\'?>\n' \
            '<definition>\n' \
            '  <datasource type="TANGO" name="%s">\n' \
            '    <device name="%s" member="attribute"' \
            ' hostname="%s" port="%s"' \
            ' group="%s"/>\n    <record name="%s"/>\n' \
            '  </datasource>\n' \
            '</definition>\n'

        commands = [
            ('nxscreate onlineds -p nxstools.xmltemplates %s %s'
             % (fname, self.flags)).split(),
            ('nxscreate onlineds --xml-package nxstools.xmltemplates %s %s'
             % (fname, self.flags)).split(),
        ]

        args = [
            ['my_test_%s' % ky, "mytest/%s/00" % ky, vl, ky]
            for ky, vl in nxstools.xmltemplates.moduleMultiAttributes.items()
        ]

        totest = []
        try:
            for ci, arg in enumerate(args):
                ds = arg[0]
                dv = arg[1]
                attr = list(arg[2])
                module = arg[3]
                if ("%s@pool" % module) in \
                   nxstools.xmltemplates.moduleMultiAttributes.keys():
                    attr.extend(
                        nxstools.xmltemplates.moduleMultiAttributes[
                            "%s@pool" % module])
                    attr.append("")
                if os.path.isfile(fname):
                    raise Exception("Test file %s exists" % fname)
                with open(fname, "w") as fl:
                    fl.write(xml % (ds, module, dv, self.host, self.port))
                try:
                    tsv = TestServerSetUp.TestServerSetUp(
                        dv, "MYTESTS1",
                        ds
                    )
                    tsv.setUp()
                    # for el in attr:
                    #     if el not in [""]:
                    #         try:
                    #             tsv.dp.CreateAttribute(el)
                    #         except Exception as e:
                    #             print(e)
                    #             tsv.dp.CreateAttribute(el)

                    skip = False
                    for el in attr:
                        if self.dsexists(
                                "%s_%s" % (ds, el.lower())
                                if el else ds):
                            skip = True
                    if not skip:
                        for el in attr:
                            totest.append(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )

                        vl, er = self.runtest(commands[ci % 2])

                        if er:
                            self.assertTrue(er.startswith(
                                "Info"))
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)
                        for el in attr:
                            dsxml = self.getds(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )
                            self.assertEqual(
                                dsout % (
                                    "%s_%s" % (ds, el.lower()) if el else ds,
                                    dv, self.host, self.port,
                                    "%s_" % (ds) if el else "__CLIENT__",
                                    el or "Value"),
                                dsxml)

                        for el in attr:
                            self.deleteds(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )
                finally:
                    os.remove(fname)
                    if tsv:
                        tsv.tearDown()
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_attributes_multi_nxsextrasp00(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = '<?xml version="1.0"?>\n' \
              '<hw>\n' \
              '<device>\n' \
              '    <name>%s</name>\n' \
              '    <type>type_tango</type>\n' \
              '    <module>%s</module>\n' \
              '    <device>%s</device>\n' \
              '    <control>tango</control>\n' \
              '    <hostname>%s:%s</hostname>\n' \
              '</device>\n' \
              '</hw>\n'

        dsout = \
            '<?xml version=\'1.0\' encoding=\'utf8\'?>\n' \
            '<definition>\n' \
            '  <datasource type="TANGO" name="%s">\n' \
            '    <device name="%s" member="attribute"' \
            ' hostname="%s" port="%s"' \
            ' group="%s"/>\n    <record name="%s"/>\n' \
            '  </datasource>\n' \
            '</definition>\n'

        if __name__ == 'test.NXSCreateOnlineDSFSTest':
            commands = [
                ('nxscreate onlineds -p test.nxsextrasp00 %s %s'
                 % (fname, self.flags)).split(),
                ('nxscreate onlineds --xml-package test.nxsextrasp00 %s %s'
                 % (fname, self.flags)).split(),
            ]
        else:
            commands = [
                ('nxscreate onlineds -p nxsextrasp00 %s %s'
                 % (fname, self.flags)).split(),
                ('nxscreate onlineds --xml-package nxsextrasp00 %s %s'
                 % (fname, self.flags)).split(),
            ]

        args = [
            ['my_test_%s' % ky, "mytest/%s/00" % ky, vl, ky]
            for ky, vl in nxsextrasp00.moduleMultiAttributes.items()
        ]

        totest = []
        try:
            for ci, arg in enumerate(args):
                ds = arg[0]
                dv = arg[1]
                attr = list(arg[2])
                module = arg[3]
                if ("%s@pool" % module) in \
                   nxsextrasp00.moduleMultiAttributes.keys():
                    attr.extend(
                        nxsextrasp00.moduleMultiAttributes[
                            "%s@pool" % module])
                    attr.append("")
                if os.path.isfile(fname):
                    raise Exception("Test file %s exists" % fname)
                with open(fname, "w") as fl:
                    fl.write(xml % (ds, module, dv, self.host, self.port))
                try:
                    tsv = TestServerSetUp.TestServerSetUp(
                        dv, "MYTESTS1",
                        ds
                    )
                    tsv.setUp()
                    # for el in attr:
                    #     if el not in [""]:
                    #         try:
                    #             tsv.dp.CreateAttribute(el)
                    #         except Exception as e:
                    #             print(e)
                    #             tsv.dp.CreateAttribute(el)

                    skip = False
                    for el in attr:
                        if self.dsexists(
                                "%s_%s" % (ds, el.lower())
                                if el else ds):
                            skip = True
                    if not skip:
                        for el in attr:
                            totest.append(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )

                        vl, er = self.runtest(commands[ci % 2])

                        if er:
                            self.assertTrue(er.startswith(
                                "Info"))
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)

                        for el in attr:
                            dsxml = self.getds(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )
                            self.assertEqual(
                                dsout % (
                                    "%s_%s" % (ds, el.lower()) if el else ds,
                                    dv,
                                    self.host, self.port,
                                    "%s_" % (ds) if el else "__CLIENT__",
                                    el or "Value"),
                                dsxml)

                        for el in attr:
                            self.deleteds(
                                "%s_%s" % (ds, el.lower())
                                if el else ds
                            )
                finally:
                    os.remove(fname)
                    if tsv:
                        tsv.tearDown()
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_onlineds_attributes_nomodule(self):
        """ test nxsccreate onlineds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
    <name>my_test_mythenroi</name>
    <type>type_tango</type>
    <module>mythenroi</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_mca8715roi</name>
    <type>type_tango</type>
    <module>mca8715roi</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_sis3302roi</name>
    <type>type_tango</type>
    <module>sis3302roi</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_sis3302</name>
    <type>type_tango</type>
    <module>sis3302</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_sis3302multiscan</name>
    <type>type_tango</type>
    <module>sis3302multiscan</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tangoattributectctrl</name>
    <type>type_tango</type>
    <module>tangoattributectctrl</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_xmcd</name>
    <type>type_tango</type>
    <module>xmcd</module>
    <device>mytest/vcexecutor/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>

</hw>
"""

        args = [
            [
                ('nxscreate onlineds %s %s'
                 % (fname, self.flags)).split(),
                [
                    'my_test_mythenroi',
                    'my_test_mca8715roi',
                    'my_test_sis3302roi',
                    'my_test_sis3302',
                    'my_test_sis3302multiscan',
                    'my_test_tangoattributectctrl',
                    'my_test_xmcd',
                ],
            ],
        ]

        totest = []
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    if er:
                        self.assertTrue(er.startswith(
                            "Info"))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)
                    for i, ds in enumerate(arg[1]):
                        self.assertTrue(
                            "SKIPPING %s:    " % (ds) in vl
                        )
        finally:
            os.remove(fname)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for SECoP """

import json
import socket
import select
import threading
from nxstools import filewriter


socketlock = threading.Lock()


class SocketHolder(object):

    """ socket holder """

    def __init__(self, sckt):
        """ constructor

        :param sckt: system socket
        :type sckt: :class:`socket.socket`
        """
        self.__sckt = sckt

    def get(self):
        """ socket getter

        :returns: system socket
        :rtype: :class:`socket.socket`
        """
        return self.__sckt

    def __del__(self):
        """  socket closer
        """
        self.__sckt.close()
        self.__sckt = None


class SecopGroup(object):

    """ socket holder """

    def __init__(self, group):
        """ constructor

        :param group: secop group name
        :type group: :obj:`str`
        """
        self.__group = group

        #: (:class:`threading.Lock`) threading lock
        self.lock = threading.Lock()
        #: (:obj:`int`) counter of steps
        self.counter = -2
        #: (:obj:`any`) any data
        self.__data = None

    def getData(self, cmd, host=None, port=None, timeout=None,
                access=None, commonblock=None):
        """ create a socket end execute the command with the group result

        :param counter: counts of scan steps
        :type counter: :obj:`int`
        :param cmd: command
        :type cmd: :obj:`str`
        :param host: secop host name
        :type host: :obj:`str`
        :param port: secop port name
        :type port: :obj:`int`
        :param timeout: minial tiemout
        :type timeout: :obj:`float`
        :param access: secop group name
        :type access: :obj:`list`< :obj:`str` or :obj:`int`>
                  or :obj:`tuple`< :obj:`str` or :obj:`int`>
        :param commonblock: common block
        :type commonblock: :obj:`dict`
        :returns: json string
        :rtype: :obj:`dict` <:obj:`str`, :obj:`any`>
        """
        counter = commonblock["__counter__"]
        data = None
        with self.lock:
            if counter == self.counter:
                data = self.data
            else:
                data = secop_cmd(cmd, host, port, timeout, commonblock)
                self.data = data
                self.counter = counter
        try:
            for idx in access:
                data = data[idx]
        except Exception:
            data = None
        return data


def secop_send(cmd, sckt, timeout=None):
    """ sends a command, reads the reply and returns a result

    :param cmd: command
    :type cmd: :obj:`str`
    :param sckt: system socket
    :type sckt: :class:`socket.socket`
    :param timeout: minial tiemout
    :type timeout: :obj:`float`
    :returns: json string
    :rtype: :obj:`dict` <:obj:`str`, :obj:`any`>
    """
    sckt.send((cmd + "\n").encode())
    sbuffer = ""
    timeout = float(timeout or 0.001)
    for tofactor in [1, 10, 100, 1000, 3000]:
        tout = tofactor * timeout
        while True:
            infds, outfds, errfds = select.select(
                [sckt], [], [], tout)
            if len(infds) == 0:
                break
            sbuffer = sbuffer + sckt.recv(1024).decode()
        try:
            l1, l2, com = sbuffer.split(' ', 2)
            argout = json.loads(com.strip())
            break
        except Exception:
            argout = sbuffer.strip()
    return argout


def secop_socket(host=None, port=None, commonblock=None):
    """ sends a command, reads the reply and returns a result

    :param host: secop host name
    :type host: :obj:`str`
    :param port: secop port name
    :type port: :obj:`int`
    :param commonblock: common block
    :type commonblock: :obj:`dict`
    :returns: system socket
    :rtype: :class:`socket.socket`
    """
    host = host or socket.gethostname()
    port = int(port or 5000)
    if not isinstance(commonblock, dict):
        sckt = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    else:
        name = "secop_%s:%s" % (host, port)
        with socketlock:
            if name in commonblock.keys() and \
               isinstance(commonblock[name], SocketHolder):
                sckt = commonblock[name].get()
            else:
                sckt = socket.socket(
                    socket.AF_INET, socket.SOCK_STREAM)
                commonblock[name] = SocketHolder(sckt)
    try:
        sckt.connect((host, port))
    except Exception as e:
        sckt.close()
        raise e
    return sckt


def secop_cmd(cmd, host=None, port=None, timeout=None, commonblock=None):
    """ create a socket end execute the command

    :param cmd: command
    :type cmd: :obj:`str`
    :param host: secop host name
    :type host: :obj:`str`
    :param port: secop port name
    :type port: :obj:`int`
    :param timeout: minial tiemout
    :type timeout: :obj:`float`
    :param commonblock: common block
    :type commonblock: :obj:`dict`
    :returns: json string
    :rtype: :obj:`dict` <:obj:`str`, :obj:`any`>
    """
    res = None
    sckt = None
    try:
        try:
            sckt = secop_socket(host, port, commonblock)
            res = secop_send(cmd, sckt, timeout)
        except Exception:
            sckt = secop_socket(host, port)
            res = secop_send(cmd, sckt, timeout)
        if not isinstance(commonblock, dict):
            sckt.close()
    except Exception:
        if sckt is not None:
            sckt.close()
    return res


def secop_group_cmd(cmd, host=None, port=None, timeout=None,
                    group=None, access=None, commonblock=None):
    """ create a socket end execute the command with the group result

    :param cmd: command
    :type cmd: :obj:`str`
    :param host: secop host name
    :type host: :obj:`str`
    :param port: secop port name
    :type port: :obj:`int`
    :param timeout: minial tiemout
    :type timeout: :obj:`float`
    :param group: secop group name
    :type group: :obj:`str`
    :param access: secop group name
    :type access: :obj:`list`< :obj:`str` or :obj:`int`>
                  or :obj:`tuple`< :obj:`str` or :obj:`int`>
    :param commonblock: common block
    :type commonblock: :obj:`dict`
    :returns: json string
    :rtype: :obj:`dict` <:obj:`str`, :obj:`any`>
    """
    try:
        res = None
        if group is None or access is None or commonblock is None:
            return secop_cmd(cmd, host, port, timeout, commonblock)
        name = "_secop_%s" % group
        sgroup = None
        with socketlock:
            if name in commonblock.keys() and \
               isinstance(commonblock[name], SecopGroup):
                sgroup = commonblock[name]
            else:
                sgroup = SecopGroup(name)
                commonblock[name] = sgroup

        res = sgroup.getData(cmd, host, port, timeout, access, commonblock)
    except Exception as e:
        res = str(e)
    return res


def create_helper_links(commonblock, meanings, entryname, samplename):
    """ create helper links

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param meanings: physical quantity name list separated by commas
    :type  meanings: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param samplename: sample name
    :type samplename: :obj:`str`
    :returns: physical quantity name list separated by commas
    :rtype: :obj:`str`
    """

    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if root.h5object.__class__.__name__ == "File":
            import nxstools.h5pywriter as nxw
        else:
            import nxstools.h5cppwriter as nxw
    else:
        raise Exception("Writer cannot be found")
    if entryname in root.names():
        en = root.open(entryname)
        if samplename in en.names():
            smp = en.open(samplename)
            for meaning in meanings.split(","):
                target, importance = get_helper_target(smp, meaning)
                if target:
                    target = "/%s/%s/%s" % (entryname, samplename, target)
                    if meaning not in smp.names():
                        nxw.link(target, smp, meaning)
    return meanings


def create_env_links(commonblock, meanings,
                     entryname, samplename):
    """ create environment links

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param meanings: physical quantity name list separated by commas
    :type  meanings: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param samplename: sample name
    :type samplename: :obj:`str`
    :returns: physical quantity name list separated by commas
    :rtype: :obj:`str`
    """

    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if root.h5object.__class__.__name__ == "File":
            import nxstools.h5pywriter as nxw
        else:
            import nxstools.h5cppwriter as nxw
    else:
        raise Exception("Writer cannot be found")
    if entryname in root.names():
        en = root.open(entryname)
        if samplename in en.names():
            smp = en.open(samplename)
            for meaning in meanings.split(","):
                target, importance = get_helper_target(smp, meaning)
                if target:
                    target = "/%s/%s/%s" % (entryname, samplename, target)
                    starget = target.split("/")
                    if "%s_env" % meaning not in smp.names():
                        env = smp.create_group(
                            "%s_env" % meaning, "NXenvironment")
                    else:
                        env = smp.open("%s_env" % meaning)
                    if starget[-2] not in env.names():
                        nxw.link("/".join(starget[:-1]), env, starget[-2])
                    if "description" not in env.names():
                        nxw.link("/".join(starget[:-2]) + "/description",
                                 env, "description")
                    if "name" not in env.names():
                        nxw.link("/".join(starget[:-2]) + "/name", env, "name")
                    if "short_name" not in env.names():
                        nxw.link("/".join(starget[:-2]) + "/short_name",
                                 env, "short_name")
                    if "type" not in env.names():
                        nxw.link("/".join(starget[:-2]) + "/type", env, "type")
    return meanings


def get_helper_target(samplegroup, meaning):
    """ get helper target

    :param samplegroup: sample group
    :type samplegroup: :obj:`str`
    :param meaning: physical quantity name
    :type  meaning: :obj:`str`
    :returns: importance
    :rtype: :obj:`int`
    """
    target = ""
    importance = -1
    for nenv in samplegroup.names():
        env = samplegroup.open(nenv)
        if "NX_class" in env.attributes.names() and  \
           "NXenvironment" == filewriter.first(
               env.attributes["NX_class"].read()):
            for nmod in env.names():
                mod = env.open(nmod)
                if "NX_class" in mod.attributes.names() and \
                   "NXsensor" == filewriter.first(
                       mod.attributes["NX_class"].read()):
                    if "measurement" in mod.names():
                        meas = mod.open("measurement")
                        imp = -1
                        if meaning == filewriter.first(meas.read()):
                            if "secop_importance" in meas.attributes.names():
                                try:
                                    imp = int(filewriter.first(
                                        meas.attributes["secop_importance"]
                                        .read()))
                                except Exception:
                                    pass
                            else:
                                imp = 0
                        if imp > importance and 'value_log' in mod.names():
                            target = "%s/%s/value_log" % (nenv, nmod)
                            importance = imp
    return target, importance


def create_sample_nxdata(commonblock, entryname, samplename,
                         dataname="data", sampledataname="data"):
    """ create sample nxsdata

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param entryname: nxentry name
    :type entryname: :obj:`str`
    :param samplename: nxsample name
    :type samplename: :obj:`str`
    :param dataname: nxdata name
    :type dataname: :obj:`str`
    :param sampledataname: sample nxdata name
    :type sampledataname: :obj:`str`
    :returns: physical quantity name list separated by commas
    :rtype: :obj:`str`
    """

    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if root.h5object.__class__.__name__ == "File":
            import nxstools.h5pywriter as nxw
        else:
            import nxstools.h5cppwriter as nxw
    else:
        raise Exception("Writer cannot be found")
    sresult = []
    if entryname in root.names():
        en = root.open(entryname)
        if dataname in en.names():
            dt = en.open(dataname)
            if samplename in en.names():
                smp = en.open(samplename)
                smppath = "/%s/%s/" % (entryname, samplename)
                smppath2 = "%s/%s/" % (entryname, samplename)

                if sampledataname in smp.names():
                    sdt = smp.open(sampledataname)
                else:
                    sdt = smp.create_group(sampledataname, "NXdata")

                for dl in nxw.get_links(dt):
                    tpath = str(dl.target_path).split(":/")[-1]
                    if tpath.startswith(smppath) \
                       or tpath.startswith(smppath2):
                        if dl.name not in sdt.names():
                            nxw.link(tpath, sdt, dl.name)
                            sresult.append(tpath)

    return ",".join(sresult)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for dalsa """


def external_data(commonblock,
                  name,
                  fileprefix,
                  filepostfix,
                  filestartnum,
                  filename):
    """ code for external_data datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: detector name
    :type name: :obj:`str`
    :param fileprefix: file prefix
    :type fileprefix: :obj:`str`
    :param filepostfix: file postfix
    :type filepostfix: :obj:`str`
    :param filestartnum: file start number
    :type filestartnum: :obj:`int`
    :param filename: file name
    :type filename: :obj:`str`
    :returns: name of saved file
    :rtype: :obj:`str`
    """
    result = ""
    try:
        filestartnum = int(filestartnum)
        if filestartnum > 0:
            filestartnum -= 1
    except Exception:
        filestartnum = 0

    if fileprefix and filepostfix:
        postfix = str(filepostfix)
        if not postfix.startswith("."):
            postfix = "." + postfix
        if postfix in [".nxs", ".nx"]:
            prefix = str(fileprefix)
            if not prefix.endswith("_"):
                prefix = prefix + "_"
            if filename:
                result = (filename).split("/")[-1].split(".")[0] + "/"

            result += name + "/" + prefix + "%06i" % filestartnum \
                + postfix + "://entry/instrument/detector"
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for datasignal """

from nxstools import filewriter

try:
    from . import mssar
except Exception:
    import mssar


def signalname(commonblock, detector, firstchannel,
               timers, mgchannels, entryname, defaultattrs=True,
               nchannels_to_skip=0, msenv=None,
               sardanasignal='SignalCounter'):
    """ code for signalname  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param detector: detector name
    :type detector: :obj:`str`
    :param firstchannel: first mg channel
    :type firstchannel: :obj:`str`
    :param timers: a list of timers separated by space
    :type timers: :obj:`str`
    :param mgchannels: a list of mgchannels separated by space
    :type mgchannels: :obj:`str`
    :param entryname:  entry group name
    :type entryname: :obj:`str`
    :param defaultattrs:  set default attributes in NXentry and NXdata
    :type defaultattrs: :obj:`bool`
    :param nchannels_to_skip: a number of mg channels to skip
    :type nchannels_to_skip: :obj:`int`
    :param nchannels_to_skip: sardana signal name
    :type nchannels_to_skip: :obj:`str`
    :param msenv: sardana environment
    :type msenv: :obj:`str`
    :param sardanasignal: signal sardana environment variable
    :type sardanasignal: :obj:`str`
    :returns: signal name
    :rtype: :obj:`str`
    """

    result = ""
    try:
        timers = [ch for ch in str(timers).split(" ") if ch]
        mgchannels = [ch for ch in str(mgchannels).split(" ") if ch]
        root = commonblock["__root__"]
        if defaultattrs:
            attrs = root.attributes
            at = attrs.create("default", "string", overwrite=True)
            at.write(entryname)
            at.close()
        nxentry = root.open(entryname)
        if defaultattrs:
            attrs = nxentry.attributes
            at = attrs.create("default", "string", overwrite=True)
            at.write("data")
            at.close()
        nxdata = nxentry.open("data")
        writer = root.parent.writer
        links = writer.get_links(nxdata)
        names = list(sorted([ch.name for ch in links]))
        sarsg = ""
        if msenv and sardanasignal:
            try:
                sarsg = mssar.mssarenv(msenv, sardanasignal)
            except Exception:
                pass
        if sarsg and sarsg in names:
            result = str(sarsg)
        elif detector in names:
            result = str(detector)
        elif firstchannel in names:
            result = str(firstchannel)
        elif mgchannels:
            for ic, ch in enumerate(mgchannels):
                if nchannels_to_skip <= ic and \
                   ch in names and ch not in timers:
                    result = str(ch)
                    break
        if not result:
            for name in names:
                fld = nxdata.open(name)
                if isinstance(fld, filewriter.FTField) and \
                   len(fld.shape) == 1 and fld.shape[0] > 1:
                    result = str(fld.name)
                    break
        if not result and names:
            result = str(names[0])
    except Exception as e:
        result = str(e)
    return result


def axesnames(commonblock, detector, firstchannel,
              timers, mgchannels, entryname, nexus_step_datasources,
              nchannels_to_skip=0, scancommand="", msenv=None,
              sardanasignal='SignalCounter', defaultattrs=False):
    """ code for axesnames  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param detector: detector name
    :type detector: :obj:`str`
    :param firstchannel: first mg channel
    :type firstchannel: :obj:`str`
    :param timers: a list of timers separated by space
    :type timers: :obj:`str`
    :param mgchannels: a list of mgchannels separated by space
    :type mgchannels: :obj:`str`
    :param entryname:  entry group name
    :type entryname: :obj:`str`
    :param nexus_step_datasources: a list of nexus_step_datasources
    :type nexus_step_datasources: :obj:`str`
    :param nchannels_to_skip: a number of mg channels to skip
    :type nchannels_to_skip: :obj:`int`
    :param scancommand: scan command
    :type scancommand: :obj:`str`
    :param msenv: sardana environment
    :type msenv: :obj:`str`
    :param sardanasignal: signal sardana environment variable
    :type sardanasignal: :obj:`str`
    :param defaultattrs:  set default attributes in NXentry and NXdata
    :type defaultattrs: :obj:`bool`
    :returns: axes names
    :rtype: :obj:`list` <:obj:`str`>
    """
    signal = signalname(
        commonblock, detector, firstchannel, timers, mgchannels, entryname,
        defaultattrs, nchannels_to_skip, msenv, sardanasignal)

    result = []
    try:
        timers = [ch for ch in str(timers).split(" ") if ch]
        stepdss = [
            ch for ch in str(nexus_step_datasources).split(" ") if ch]
        root = commonblock["__root__"]
        nxentry = root.open(entryname)
        nxdata = nxentry.open("data")
        writer = root.parent.writer
        links = writer.get_links(nxdata)
        names = list(sorted([ch.name for ch in links]))
        if scancommand:
            pars = [par for par in str(scancommand).split(" ") if par]
            if len(pars) > 1:
                scanaxis = pars[1]
                if scanaxis in names:
                    if scanaxis in stepdss:
                        stepdss.remove(scanaxis)
                    stepdss.insert(0, scanaxis)
        if signal != detector:
            dt = nxdata.open(signal)
            dtshape = dt.shape
            if len(dtshape) > 0 and dtshape[0] > 0:
                if stepdss and stepdss[0] in names:
                    ax = nxdata.open(stepdss[0])
                    if len(ax.shape) == 1 and ax.shape[0] == dtshape[0]:
                        result = [stepdss[0]]
            if result and len(dtshape) > 1 and dtshape[1] > 0:
                if len(stepdss) > 1 and stepdss[1] in names:
                    ax = nxdata.open(stepdss[1])
                    if len(ax.shape) == 1 and ax.shape[0] == dtshape[1]:
                        result.append(stepdss[1])
            if result:
                while len(dtshape) > len(result):
                    result.append(".")
    except Exception as e:
        result = str(e)
    if not result:
        result = ["."]
    return result



#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

""" NDTS TANGO device tools """

import sys
import os
import time
import socket

#: (:obj:`dict` <:obj:`str` , :obj:`dict` <:obj:`str` , :obj:`str` > >)
#:     standard component template variables
#:     and its [default value, doc string]
standardComponentVariables = {}
#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     xml template files of modules
standardComponentTemplateFiles = {}
#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     xml template files of modules
moduleTemplateFiles = {}
#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     important attributes of modules
moduleMultiAttributes = {}


class PackageHandler(object):

    """ xml templates package loader
    """

    def __init__(self, packagename='nxstools.xmltemplates'):
        """ constructor

        :param packagename: full package name
        :type packagename: :obj:`str`
        """
        self.packagename = None
        self.package = None
        self.packagepath = None
        self.loadXMLTemplates(packagename)

    def loadXMLTemplates(self, packagename):
        """ load xml template module variables

        :param packagename: full package name
        :type packagename: :obj:`str`
        """
        self.packagename = packagename
        global standardComponentVariables
        global standardComponentTemplateFiles
        global moduleTemplateFiles
        global moduleMultiAttributes
        self.package = __import__(
            packagename, globals(), locals(), packagename[-1])
        standardComponentVariables = \
            self.package.standardComponentVariables
        standardComponentTemplateFiles = \
            self.package.standardComponentTemplateFiles
        moduleTemplateFiles = \
            self.package.moduleTemplateFiles
        moduleMultiAttributes = \
            self.package.moduleMultiAttributes
        self.packagepath = os.path.dirname(self.package.__file__)


#: (:class:`PackageHandler`) xml template package handler
xmlPackageHandler = PackageHandler()


#: attributes of device modules to acquire with elements:
#  'module': [<sardana_pool_attr>, <original_tango_attr>]
moduleAttributes = {
    'counter_tango': ['Value', 'Counts'],
    'v260': ['Value', 'Counts'],
    'dgg2': ['Value', 'SampleTime'],
    'mca_8701': ['Value', 'Data'],
    'mca_sis3302new': ['Value', 'Data'],
    'mca_sis3302': ['Value', 'Data'],
    'mythenroi': ['Value', None],
    'mca8715roi': ['Value', None],
    'sis3302roi': ['Value', None],
    'sis3302': ['Value', None],
    'sis3302multiscan': ['Value', None],
    'sis3610': ['Value', 'Value'],
    'vdot32in': ['Value', 'Value'],
    'sis3820': ['Value', 'Counts'],
    'tangoattributectctrl': ['Value', None],
    'tip551': ['Value', 'Voltage'],
    'tip850dac': ['Value', 'Voltage'],
    'tip830': ['Value', 'Counts'],
    'tip850adc': ['Value', 'Value'],
    'vfcadc': ['Value', 'Counts'],
    'xmcd': ['Value', None],
}


#: (:obj:`list` <:obj:`str`>) modules of 2d detectors
twoDModules = [
    'pilatus100k', 'pilatus300k', 'pilatus1m',
    'pilatus2m', 'pilatus6m', 'pco4000', 'perkinelmerdetector',
    'lambda', 'pedetector', 'perkinelmer',
    'pco', 'pcoedge', 'marccd', 'perkinelmer',
    #
    'lcxcamera', 'limaccd', 'eigerpsi',
    'eigerdectris'
]


#: (:obj:`list` <:obj:`str`>) modules of motors
motorModules = [
    'absbox', 'motor_tango', 'kohzu', 'smchydra', 'lom', 'oms58', 'e6c',
    'omsmaxv', 'spk', 'pie710', 'pie712', 'e6c_p09_eh2', 'smaract'
    #
    #    'analyzerep01', 'tth', 'atto300',  'phaseretarder',
    #    'hexa',
    #    'tm', 'cube', , 'piezonv40', 'smaractmcs',
    #    'slt', 'bscryotempcontrolp01',
    #    'dcm_energy', 'elom', 'diffracmu',  'tcpipmotor',
    #    'galil_dmc', 'pico8742', 'oxfcryo700ctrl', 'analyzer', 'nfpaxis',
    #    'smarpod', 'mult',
    #
    #    'oxfcryo700',
]

#: (:obj:`list` <:obj:`str`>) counter/timer modules
ctModules = [
    'mca8715roi', 'onedroi', 'sis3820', 'sis3302roi',
    'xmcd', 'vfcadc', 'mythenroi', 'mhzdaqp01', 'dgg2',
    'tangoattributectctrl'
]

#: (:obj:`list` <:obj:`str`>) modules of 0D detectors
zeroDModules = ['tip830']

#: (:obj:`list` <:obj:`str`>) modules of 1D detectors
oneDModules = ['mca_xia']

#: (:obj:`list` <:obj:`str`>) IO register modules
ioRegModules = ['sis3610']


#: (:obj:`bool`) True if tango available
PYTANGO = False
try:
    try:
        import tango
    except Exception:
        import PyTango as tango
    PYTANGO = True
except Exception:
    pass


for mn in motorModules:
    moduleAttributes[mn] = ['Position', 'Position']

for nm in ctModules + zeroDModules + oneDModules:
    if nm not in moduleAttributes:
        moduleAttributes[nm] = ['Value', None]


def generateDeviceNames(prefix, first, last, minimal=False):
    """ generates device names

    :param prefix: device name prefix
    :type prefix: :obj:`str`
    :param first: first device index
    :type first: :obj:`int`
    :param last: last device index
    :type last: :obj:`int`
    :returns: device names
    :rtype: :obj:`list` <:obj:`str`>
    """
    names = []
    if prefix.strip():
        for i in range(first, last + 1):
            if not minimal:
                names.append(prefix + ("0" if len(str(i)) == 1 else "")
                             + str(i))
            else:
                names.append(prefix + str(i))
    return names


def getAttributes(device, host=None, port=10000):
    """ provides a list of device attributes

    :param device: tango device name
    :type device: :obj:`str`
    :param host: device host
    :type host: :obj:`str`
    :param port: device port
    :type port: :obj:`int`
    :returns: list of device attributes
    :rtype: :obj:`list` <:obj:`str`>
    """
    if host:
        dp = tango.DeviceProxy("%s:%s/%s" % (host, port, device))
    else:
        dp = tango.DeviceProxy(device)
    attr = dp.attribute_list_query()
    return [at.name for at in attr if at.name not in ['State', 'Status']]


def openServer(device):
    """ opens connection to the configuration server

    :param configuration: server device name
    :type configuration: :obj:`str`
    :returns: configuration server proxy
    :rtype: :class:`tango.DeviceProxy`
    """
    found = False
    cnt = 0
    # spliting character
    try:
        #: configuration server proxy
        cnfServer = tango.DeviceProxy(device)
    except tango.DevFailed:
        found = True

    if found:
        sys.stderr.write(
            "Error: Cannot connect to the server: %s\n" % device)
        sys.stderr.flush()
        sys.exit(0)

    while not found and cnt < 1000:
        if cnt > 1:
            time.sleep(0.02)
        try:
            if cnfServer.state() != tango.DevState.RUNNING:
                found = True
        except tango.DevFailed:
            time.sleep(0.02)
            found = False
        cnt += 1

    if not found:
        sys.stderr.write("Error: Setting up %s takes to long\n" % device)
        sys.stderr.flush()
        sys.exit(0)

    return cnfServer


def storeDataSource(name, xml, server):
    """ stores datasources in Configuration Server

    :param name: datasource name
    :type name: :obj:`str`
    :param xml: datasource xml string
    :type xml: :obj:`str`
    :param server: configuration server
    :type server: :obj:`str`
    """
    proxy = openServer(server)
    proxy.Open()
    proxy.XMLString = str(xml)
    proxy.StoreDataSource(str(name))


def getServerTangoHost(server):
    """ fetches the server tango_host:tango_port

    :param server: tango server
    :type server: :obj:`str`
    :returns: tango host
    :rtype: :obj:`str`
    """
    if server:
        proxy = openServer(server)
        host = proxy.get_db_host()
        port = proxy.get_db_port()
    else:
        db = tango.Database()
        host = db.get_db_host().split(".")[0]
        port = db.get_db_port()
    shost = str(host).split(".")
    if len(shost) > 0:
        host = shost[0]
    return "%s:%s" % (host, port)


def getDataSourceComponents(server, verbose=False):
    """ gets datasource components

    :param server: configuration server
    :type server: :obj:`str`
    :param verbose: additional printouts
    :type verbose: :obj:`bool`
    :returns: dictionary with datasource components
    :rtype: :obj:`dict` <:obj:`str`, :obj:`list` <:obj:`str`>>
    """
    dscps = {}
    proxy = openServer(server)
    proxy.Open()
    acps = proxy.availableComponents()
    errcps = []
    for cp in acps:
        try:
            depcps = proxy.dependentComponents([cp])
            for dcp in depcps:
                dss = proxy.componentDataSources(dcp)
                for ds in dss:
                    if ds not in dscps:
                        dscps[ds] = []
                    if cp not in dscps[ds]:
                        dscps[ds].append(cp)
        except Exception as e:
            errcps.append(cp)
            if verbose:
                sys.stderr.write(str(e))
                sys.stderr.write(
                    "Error: Internal error of the %s component\n" % cp)
                sys.stderr.flush()
    if not verbose and errcps:
        sys.stderr.write(
            "Info: Inconsistent components is the NeXus database - %s\n"
            % ", ".join(errcps))
        sys.stderr.flush()

    return dscps


def storeComponent(name, xml, server, mandatory=False):
    """ stores components in Configuration Server

    :param name: component name
    :type name: :obj:`str`
    :param xml: component xml string
    :type xml: :obj:`str`
    :param server: configuration server
    :type server: :obj:`str`
    :param mandatory: set component as mandatory
    :type mandatory: :obj:`bool`
    """
    proxy = openServer(server)
    proxy.Open()
    proxy.XMLString = str(xml)
    proxy.StoreComponent(str(name))
    if mandatory:
        proxy.SetMandatoryComponents([str(name)])


def getClassName(devicename):
    """ provides device class name

    :param devicename: device name
    :type devicename: :obj:`str`
    :returns: class name
    :rtype: :obj:`str`
   """
    try:
        db = tango.Database()
    except Exception:
        sys.stderr.write(
            "Info: Cannot connect to %s" % devicename
            + " on host %s\n" % os.environ['TANGO_HOST'])
        sys.stderr.flush()
        return ""

    return db.get_class_for_device(devicename)


def getServers(name='NXSConfigServer'):
    """ provides server device names

    :param name: server instance name
    :type name: :obj:`str`
    :returns: list of the server device names
    :rtype: :obj:`list` <:obj:`str`>
    """

    try:
        db = tango.Database()
    except Exception:
        sys.stderr.write(
            "Error: Cannot connect to %s" % name
            + " on host %s\n" % os.environ['TANGO_HOST'])
        sys.stderr.flush()
        return ""

    servers = db.get_device_exported_for_class(name).value_string
    return servers


def _remoteCall(server, func, *args, **kwargs):
    """ executes function on remove tango host db setup

    :param server: remove tango server device name
    :type server: :obj:`str`
    :param func: executed function
    :type func: :obj:`instancemethod`
    :param args: function list arguments
    :type args: :obj:`list` <`and`>
    :param kwargs: function dict arguments
    :type kwargs: :obj:`dict` <:obj:`str` , `any`>
    :returns: function result
    :rtype: `any`
    """
    lserver = None
    localtango = None
    if server and ":" in server and server.strip():
        lserver = server.split("/")[0]
    if lserver:
        lserver = lserver.strip()
    if lserver:
        if ":" not in lserver:
            lserver = lserver + ":10000"
        localtango = os.environ.get('TANGO_HOST')
        os.environ['TANGO_HOST'] = lserver

    res = func(*args, **kwargs)
    if lserver:
        if localtango is not None:
            os.environ['TANGO_HOST'] = localtango
        else:
            os.environ.pop('TANGO_HOST')
    return res


def listServers(server, name='NXSConfigServer'):
    """ finds server names

    :param name: server instance name
    :type name: :obj:`str`
    :returns: server list
    :rtype: :obj:`list` <:obj:`str`>
    """
    return _remoteCall(server, getServers, name)


def findClassName(server, name):
    """ finds class name

    :param name: device name
    :type name: :obj:`str`
    :returns: class name
    :rtype: :obj:`str`
    """
    return _remoteCall(server, getClassName, name)


def checkServer(name='NXSConfigServer'):
    """ provides server device name if only one or error in the other case

    :param name: server name
    :type name: :obj:`str`
    :returns: server device name or empty string if error appears
    :rtype: :obj:`str`
    """
    servers = getServers(name)
    if not servers:
        sys.stderr.write(
            "Info: NeXus hasn't been setup yet. \n\n")
        sys.stderr.flush()
        return ""
    if len(servers) > 1:
        thost = os.getenv('TANGO_HOST')
        if thost:
            tlhost = thost.split(":")[0]
            tservers = [sr for sr in servers if sr.endswith("/%s" % tlhost)]
            if tservers:
                servers = tservers
            else:
                lhost = socket.gethostname()
                lservers = [sr for sr in servers if sr.endswith("/%s" % lhost)]
                if lservers:
                    servers = lservers
        if len(servers) > 1:
            sys.stderr.write(
                "Info: More than on %s " % name
                + "on the current host running. \n\n"
                + "    Please specify the server:"
                + "\n        %s\n\n" % "\n        ".join(servers))
            sys.stderr.flush()
            return ""
    return servers[0]


if __name__ == "__main__":
    pass



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for pilc """

# import json
try:
    import tango
except Exception:
    import PyTango as tango


def triggermode_cb(commonblock, name, triggermode,
                   nbtriggers, triggersperfile,
                   hostname, device,
                   filename, entryname,
                   insname, pilcfileprefix, pilcfiledir,
                   timeid=False):
    """ code for triggermode_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: component name
    :type name: :obj:`str`
    :param triggermode:  trigger mode
    :type triggermode: :obj:`int` or :obj:`str`
    :param nbtriggers: a number of triggers
    :type nbtriggers: :obj:`int`
    :param triggersperfile: a number of triggers per file
    :type triggersperfile: :obj:`int`
    :param hostname: tango host name
    :type hostname: :obj:`str`
    :param device: tango device name
    :type device: :obj:`str`
    :param filename: master file name
    :type filename: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param insname: instrument name
    :type insname: :obj:`str`
    :param pilcfileprefix: pilc file name prefix
    :type pilcfileprefix :obj:`str`
    :param pilcfiledir: pilc file name directory
    :type pilcfiledir :obj:`str`
    :param timeid: file id with timestamp
    :type timeid: :obj:`bool`
    :returns: triggermode
    :rtype: :obj:`str` or :obj:`int`
    """

    dp = tango.DeviceProxy('%s/%s' % (hostname, device))
    fpattern = pilcfileprefix.split("/")[-1]
    if triggersperfile >= 1:
        nbfiles = (nbtriggers + triggersperfile - 1) // triggersperfile
    else:
        nbfiles = 1
    fields = ["counter", "time", "trigger",
              "encoder_1", "encoder_2",
              "encoder_3", "encoder_4", "encoder_5"]
    nbstart = 0
    filepostfix = ".nxs"
    try:
        pilcfilename = str(dp.FileName)
        if pilcfilename.startswith(fpattern + "_") and \
                pilcfilename.endswith(filepostfix):
            nblast = int(pilcfilename[
                    len(fpattern) + 1: - len(filepostfix)])
            nbstart = max(0, nblast - nbfiles + 1)
    except Exception:
        pilcfilename = None
    nblist = [nb for nb in range(nbstart, nbstart + nbfiles)]

    if filename:
        path = (filename).split("/")[-1].split(".")[0] + "/"
    else:
        path = ""
    path += '%s/%s_' % (name, fpattern)
    result = triggermode

    spf = 0
    cfid = 0
    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if hasattr(root, "currentfileid") and hasattr(root, "stepsperfile"):
            spf = root.stepsperfile
            cfid = root.currentfileid
        if root.h5object.__class__.__name__ == "File":
            import nxstools.h5pywriter as nxw
        else:
            import nxstools.h5cppwriter as nxw
    else:
        raise Exception("Writer cannot be found")

    en = root.open(entryname)
    dt = en.open("data")
    ins = en.open(insname)
    pilc = ins.open(name)
    pilcdata = pilc.open("data")
    col = pilc.open("collection")

    if timeid and pilcfilename and "filenames" in col.names():
        try:
            filenames = list(col.open("filenames").read())
            if filenames:
                filenames.append(str(pilcfilename))
                nlist = []
                for fname in filenames:
                    fname = str(fname)
                    if fname.startswith(fpattern + "_") and \
                            fname.endswith(filepostfix):
                        nlist.append(int(fname[
                            len(fpattern) + 1: - len(filepostfix)]))
                nblist = nlist
        except Exception:
            pass
    for nbf in nblist:
        # nnbf = "%05i" % (nbf)
        for field in fields:
            fnbf = "%s_%05i" % (field, nbf)
            if spf > 0 and cfid > 0:
                if cfid == nbf:
                    nxw.link(
                        "%s%05i%s://entry/data/%s"
                        % (path, nbf, filepostfix, field),
                        pilcdata, field)
                    nxw.link("/%s/%s/%s/data/%s"
                             % (entryname, insname, name, field),
                             dt, "%s_%s" % (name, field))
                nxw.link("%s%05i%s://entry/data/%s"
                         % (path, nbf, filepostfix, field),
                         pilcdata, "%s" % (fnbf))
            else:
                nxw.link("%s%05i%s://entry/data/%s"
                         % (path, nbf, filepostfix, field),
                         pilcdata, "%s" % (fnbf))
                nxw.link("/%s/%s/%s/data/%s" %
                         (entryname, insname, name, fnbf), dt,
                         "%s_%s" % (name, fnbf))
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfiguratorTest.py
# unittests for field Tags running Tango Server
#
import unittest
import os
import sys
import random
import struct
import binascii
import time
from nxstools import nxscreate


try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO


if sys.version_info > (3,):
    unicode = str
    long = int


# if 64-bit machione
IS64BIT = (struct.calcsize("P") == 8)


# test fixture
class NXSCreateClientDSFSTest(unittest.TestCase):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        unittest.TestCase.__init__(self, methodName)

        self.helperror = "Error: too few arguments\n"

        self.helpinfo = """usage: nxscreate [-h]
                 {clientds,tangods,deviceds,onlinecp,onlineds,poolds,stdcomp,comp,compare}
                 ...

 Command-line tool for creating NXSConfigServer configuration of Nexus Files

positional arguments:
  {clientds,tangods,deviceds,onlinecp,onlineds,poolds,stdcomp,comp,compare}
                        sub-command help
    clientds            create client datasources
    tangods             create tango datasources
    deviceds            create datasources for all device attributes
    onlinecp            create component from online.xml file
    onlineds            create datasources from online.xml file
    poolds              create datasources from sardana pool device
    stdcomp             create component from the standard templates
    comp                create simple components
    compare             compare two online.xml files

optional arguments:
  -h, --help            show this help message and exit

For more help:
  nxscreate <sub-command> -h

"""

        try:
            # random seed
            self.seed = long(binascii.hexlify(os.urandom(16)), 16)
        except NotImplementedError:
            # random seed
            self.seed = long(time.time() * 256)  # use fractional seconds

        self._rnd = random.Random(self.seed)

        self._bint = "int64" if IS64BIT else "int32"
        self._buint = "uint64" if IS64BIT else "uint32"
        self._bfloat = "float64" if IS64BIT else "float32"

        # home = expanduser("~")

        self.directory = "."
        self.flags = ""

    # sets xmlconfiguration
    # \param xmlc configuration instance
    # \param xml xml configuration string
    def setXML(self, xmlc, xml):
        xmlc.XMLString = xml

    # gets xmlconfiguration
    # \param xmlc configuration instance
    # \returns xml configuration string
    def getXML(self, xmlc):
        return xmlc.XMLString

    # test starter
    # \brief Common set up
    def setUp(self):
        print("\nsetting up...")
        print("SEED = %s" % self.seed)

    # test closer
    # \brief Common tear down
    def tearDown(self):
        print("tearing down ...")

    def dsexists(self, name):
        return os.path.isfile("%s/%s.ds.xml" % (self.directory, name))

    def cpexists(self, name):
        return os.path.isfile("%s/%s.xml" % (self.directory, name))

    def getds(self, name):
        with open("%s/%s.ds.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def getcp(self, name):
        with open("%s/%s.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def deleteds(self, name):
        os.remove("%s/%s.ds.xml" % (self.directory, name))

    def deletecp(self, name):
        os.remove("%s/%s.xml" % (self.directory, name))

    def runtest(self, argv):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        nxscreate.main()
        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    def runtestexcept(self, argv, exception):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        try:
            error = False
            nxscreate.main()
        except exception:
            error = True
        self.assertEqual(error, True)

        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    # Exception tester
    # \param exception expected exception
    # \param method called method
    # \param args list with method arguments
    # \param kwargs dictionary with method arguments
    def myAssertRaise(self, exception, method, *args, **kwargs):
        try:
            error = False
            method(*args, **kwargs)
        except exception:
            error = True
        self.assertEqual(error, True)

    # sets selection configuration
    # \param selectionc configuration instance
    # \param selection selection configuration string
    def setSelection(self, selectionc, selection):
        selectionc.selection = selection

    # gets selectionconfiguration
    # \param selectionc configuration instance
    # \returns selection configuration string
    def getSelection(self, selectionc):
        return selectionc.selection

    def test_clientds_simple(self):
        """ test nxsccreate clientds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate clientds starttimetest %s' % self.flags).split(),
                'starttimetest',
                """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="starttimetest">
    <record name="starttimetest"/>
  </datasource>
</definition>
"""
            ],
            [
                ('nxscreate clientds endtimetest %s' % self.flags).split(),
                'endtimetest',
                """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="endtimetest">
    <record name="endtimetest"/>
  </datasource>
</definition>
"""
            ],
            [
                ('nxscreate clientds wwwtest %s' % self.flags).split(),
                'wwwtest',
                """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="wwwtest">
    <record name="wwwtest"/>
  </datasource>
</definition>
"""
            ],
            [
                ('nxscreate clientds abstest %s' % self.flags).split(),
                'abstest',
                """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="abstest">
    <record name="abstest"/>
  </datasource>
</definition>
"""
            ],
        ]

        totest = []
        try:
            for arg in args:
                if not self.dsexists(arg[1]):
                    totest.append(arg[1])

                    vl, er = self.runtest(arg[0])

                    self.assertEqual('', er)
                    self.assertTrue(vl)
                    xml = self.getds(arg[1])
                    self.assertEqual(arg[2], xml)

                    self.deleteds(arg[1])
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_clientds_first_last(self):
        """ test nxsccreate clientds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate clientds -v test_exp_c  -l3 %s'
                 % self.flags).split(),
                ['test_exp_c01',
                 'test_exp_c02',
                 'test_exp_c03'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_c01">
    <record name="test_exp_c01"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_c02">
    <record name="test_exp_c02"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_c03">
    <record name="test_exp_c03"/>
  </datasource>
</definition>
""",
                ],
            ],
            [
                ('nxscreate clientds -v test_exp_mot  -f2 -l3 %s'
                 % self.flags).split(),
                ['test_exp_mot02',
                 'test_exp_mot03'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_mot02">
    <record name="test_exp_mot02"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_mot03">
    <record name="test_exp_mot03"/>
  </datasource>
</definition>
""",
                ],
            ],
            [
                ('nxscreate clientds --device test_exp_vfc'
                 ' --first 2 --last 3 %s' % self.flags).split(),
                ['test_exp_vfc02',
                 'test_exp_vfc03'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vfc02">
    <record name="test_exp_vfc02"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vfc03">
    <record name="test_exp_vfc03"/>
  </datasource>
</definition>
""",
                ],
            ],
        ]

        totest = []
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_clientds_minimal(self):
        """ test nxsccreate clientds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate clientds -v test_exp_cc  -m -f3 -l5 %s'
                 % self.flags).split(),
                ['test_exp_cc3',
                 'test_exp_cc4',
                 'test_exp_cc5'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_cc3">
    <record name="test_exp_cc3"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_cc4">
    <record name="test_exp_cc4"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_cc5">
    <record name="test_exp_cc5"/>
  </datasource>
</definition>
""",
                ],
            ],
            [
                ('nxscreate clientds -v test_exp_dd '
                 '--minimal-device --first 3 --last 4 %s'
                 % self.flags).split(),
                ['test_exp_dd3',
                 'test_exp_dd4'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_dd3">
    <record name="test_exp_dd3"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_dd4">
    <record name="test_exp_dd4"/>
  </datasource>
</definition>
""",
                ],
            ],
        ]

        totest = []
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_clientds_source_prefix(self):
        """ test nxsccreate clientds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate clientds -v testcounter -s test_exp_vc  -f3 -l5 %s'
                 % self.flags).split(),
                ['test_exp_vc03',
                 'test_exp_vc04',
                 'test_exp_vc05'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc03">
    <record name="testcounter03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc04">
    <record name="testcounter04"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc05">
    <record name="testcounter05"/>
  </datasource>
</definition>
""",
                ],
            ],
            [
                ('nxscreate clientds --device testdec '
                 '--datasource-prefix test_exp_d '
                 '--first 3 --last 4 %s'
                 % self.flags).split(),
                ['test_exp_d03',
                 'test_exp_d04'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d03">
    <record name="testdec03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d04">
    <record name="testdec04"/>
  </datasource>
</definition>
""",
                ],
            ],
        ]

        totest = []
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_clientds_overwrite_false(self):
        """ test nxsccreate clientds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate clientds -v testcounter -s test_exp_vc  -f3 -l5 %s'
                 % self.flags).split(),
                ['test_exp_vc03',
                 'test_exp_vc04',
                 'test_exp_vc05'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc03">
    <record name="testcounter03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc04">
    <record name="testcounter04"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc05">
    <record name="testcounter05"/>
  </datasource>
</definition>
""",
                ],
                ('nxscreate clientds -v test2counter -s test_exp_vc -f3 -l5 %s'
                 % self.flags).split(),
            ],
            [
                ('nxscreate clientds --device testdec '
                 '--datasource-prefix test_exp_d '
                 '--first 3 --last 4 %s'
                 % self.flags).split(),
                ['test_exp_d03',
                 'test_exp_d04'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d03">
    <record name="testdec03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d04">
    <record name="testdec04"/>
  </datasource>
</definition>
""",
                ],
                ('nxscreate clientds --device test2dec '
                 '--datasource-prefix test_exp_d '
                 '--first 3 --last 4 %s'
                 % self.flags).split(),
            ],
        ]

        totest = []
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    self.assertEqual('', er)
                    self.assertTrue(vl)
                    vl, er = self.runtestexcept(arg[3], Exception)

                    self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_clientds_overwrite_true(self):
        """ test nxsccreate clientds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate clientds -v testcounter -s test_exp_vc  -f3 -l5 %s'
                 % self.flags).split(),
                ['test_exp_vc03',
                 'test_exp_vc04',
                 'test_exp_vc05'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc03">
    <record name="testcounter03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc04">
    <record name="testcounter04"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc05">
    <record name="testcounter05"/>
  </datasource>
</definition>
""",
                ],
                ('nxscreate clientds -v test2counter -o -s test_exp_vc '
                 ' -f3 -l5 %s' % self.flags).split(),
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc03">
    <record name="test2counter03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc04">
    <record name="test2counter04"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_vc05">
    <record name="test2counter05"/>
  </datasource>
</definition>
""",
                ],
            ],
            [
                ('nxscreate clientds --device testdec '
                 '--datasource-prefix test_exp_d '
                 '--first 3 --last 4 %s'
                 % self.flags).split(),
                ['test_exp_d03',
                 'test_exp_d04'],
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d03">
    <record name="testdec03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d04">
    <record name="testdec04"/>
  </datasource>
</definition>
""",
                ],
                ('nxscreate clientds --device test2dec --overwrite '
                 '--datasource-prefix test_exp_d '
                 '--first 3 --last 4 %s'
                 % self.flags).split(),
                [
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d03">
    <record name="test2dec03"/>
  </datasource>
</definition>
""",
                    """<?xml version='1.0' encoding='utf8'?>
<definition>
  <datasource type="CLIENT" name="test_exp_d04">
    <record name="test2dec04"/>
  </datasource>
</definition>
""",
                ],
            ],
        ]

        totest = []
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])

                    self.assertEqual('', er)
                    self.assertTrue(vl)
                    vl, er = self.runtest(arg[3])

                    self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[4][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)


if __name__ == '__main__':
    unittest.main()
